# Generated by Django 2.2.17 on 2021-01-25 18:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp', '0014_auto_20200909_2206'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='activity',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control']},
        ),
        migrations.AlterModelOptions(
            name='follower',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control']},
        ),
        migrations.AlterModelOptions(
            name='ldpsource',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'ordering': ('federation',)},
        ),
        migrations.AlterModelOptions(
            name='scheduledactivity',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control']},
        ),
    ]
